const std = @import("std");
const mach = @import("mach");

pub fn build(b: *std.Build) !void {
    const target = b.standardTargetOptions(.{});
    const optimize = b.standardOptimizeOption(.{});

    const mach_dep = b.dependency("mach", .{
        .target = target,
        .optimize = optimize,
        // Only using mach.core, avoid pulling unnecessary dependencies.
        .core = true,
    });

    const zmath = b.dependency("zmath", .{ .target = target, .optimize = optimize });
    const zigimg = b.dependency("zigimg", .{ .target = target, .optimize = optimize });
    const flecszigble = b.dependency("flecs-zig-ble", .{ .target = target, .optimize = optimize });

    const app = try mach.CoreApp.init(b, mach_dep.builder, .{
        .name = "zig-bloxel-game",
        .src = "src/main.zig",
        .target = target,
        .optimize = optimize,
        .deps = &.{
            .{ .name = "zmath", .module = zmath.module("root") },
            .{ .name = "zigimg", .module = zigimg.module("zigimg") },
            .{ .name = "flecs-zig-ble", .module = flecszigble.module("flecs-zig-ble") },
        },
    });
    // Honestly, not sure how this works, but this seems
    // to be required for Flecs' header file to be found?
    app.compile.linkLibrary(flecszigble.artifact("flecs-zig-ble"));
    if (b.args) |args| app.run.addArgs(args);

    const run_step = b.step("run", "Run the app");
    run_step.dependOn(&app.run.step);

    const unit_tests = b.addTest(.{
        .root_source_file = .{ .path = "src/main.zig" },
        .target = target,
        .optimize = optimize,
    });

    const run_unit_tests = b.addRunArtifact(unit_tests);
    const test_step = b.step("test", "Run unit tests");
    test_step.dependOn(&run_unit_tests.step);
}
